/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.core.impl.context;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.core.session.ILocaleConfig;
import org.ow2.opensuit.core.session.PageContext;
import org.ow2.opensuit.xml.base.Application;


public class OpenSuitSessionImpl extends OpenSuitSession
{
	private Application _site;
//	private Locale _locale;
	private ILocaleConfig localeConfig;
	private PageContext _context;
	
	public static void enterRequest(HttpServletRequest iRequest, HttpServletResponse iResponse)
	{
		_currentRequest.set(iRequest);
		_currentResponse.set(iResponse);
	}
	public static void exitRequest()
	{
		_currentRequest.remove();
		_currentResponse.remove();
	}

	public static OpenSuitSessionImpl getSession(HttpServletRequest iRequest)
	{
		return (OpenSuitSessionImpl)iRequest.getSession().getAttribute("_OPENSUIT_CTX_");
	}
	public static void setSession(HttpServletRequest iRequest, OpenSuitSessionImpl iSession)
	{
		iRequest.getSession().setAttribute("_OPENSUIT_CTX_", iSession);
	}
	public OpenSuitSessionImpl(Application iSite)
	{
		_site = iSite;
	}
	/*
	public void setLocale(Locale locale)
	{
		_locale = locale;
	}
	public Locale getLocale()
	{
		return _locale;
	}
	*/
	@Override
	public Locale getLocale()
	{
		if(localeConfig == null)
			return null;
	    return localeConfig.getLocale();
	}
	public Application getApplication()
	{
	    return _site;
	}
	public ILocaleConfig getLocaleConfig()
    {
    	return localeConfig;
    }
	public void setLocaleConfig(ILocaleConfig localeConfig)
    {
    	this.localeConfig = localeConfig;
    }
	
	// ================================================================
	// === Page FormRenderingContext Attributes
	// ================================================================
	private Hashtable _name2Attr = new Hashtable();
	
	public Object getAttribute(String iName)
	{
	    return _name2Attr.get(iName);
	}
	public Enumeration getAttributeNames()
	{
	    return _name2Attr.keys();
	}
	public void removeAttribute(String iName)
	{
		_name2Attr.remove(iName);
	}
	public void setAttribute(String iName, Object iValue)
	{
		_name2Attr.put(iName, iValue);
	}
	
	
	public PageContext getCurrentPageContext()
	{
		return _context;
	}
	public void setCurrentPageContext(PageContext iContext)
	{
		_context = iContext;
	}


}
