/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.html.form.field;


import javax.servlet.http.HttpServletRequest;

import org.ow2.opensuit.core.validation.ValidationError;
import org.ow2.opensuit.xml.base.binding.Expression;
import org.ow2.opensuit.xml.base.html.form.FormSubmitContext;
import org.ow2.opensuit.xml.base.validation.IValidator;
import org.ow2.opensuit.xml.base.validation.Mandatory;
import org.ow2.opensuit.xmlmap.annotations.XmlChild;
import org.ow2.opensuit.xmlmap.annotations.XmlChildren;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;


@XmlDoc("The base class for single valued form fields.")
@XmlElement
public abstract class BaseSingleValueField extends BaseEditableField implements IValidable, IInitializable
{
	@XmlDoc("The expression to get (and possibly set) the associated value.")
	@XmlChild(name="Bind")
	protected Expression bind;
	
	@XmlDoc("Validators associated to this field.")
	@XmlChildren(name="Validators", minOccurs=0)
	protected IValidator[] validators;
	
	protected abstract Object getValueFromRequest(HttpServletRequest iRequest, FormSubmitContext iSubmitContext) throws ValidationError;
	
	public String getFieldName()
	{
		return bind.getID();
	}
	
    public boolean isMandatory(HttpServletRequest request) 
    {
        if (validators != null) {
            for (IValidator validator : validators) {
                if (validator instanceof Mandatory) {
                    return true;
                }
            }
        }
        return false;
    }

	@Override
	public boolean isEditable(HttpServletRequest iRequest)
	{
		return bind != null && !bind.isReadOnly() && super.isEditable(iRequest);
	}
	public void submit(HttpServletRequest iRequest, FormSubmitContext iSubmitContext) throws Exception
	{
		if(!isEditable(iRequest))
			return;
		
		// --- parse value (from request)
		Object value = null;
		boolean validate = true;
		try
		{
			value = getValueFromRequest(iRequest, iSubmitContext);
		}
		catch(ValidationError e)
		{
			// --- formatting error stops further validation
			e.setStopValidation(true);
			validate = false;
			iSubmitContext.getValidationErrors().addItemError(getFieldName(), e);
		}
		
		// --- set
		bind.set(iRequest, value);
		
		// --- validate
		if(validate && validators != null)
		{
			for(int i=0; i<validators.length; i++)
			{
				try
				{
					validators[i].checkValidity(value);
				}
				catch(ValidationError e)
				{
					iSubmitContext.getValidationErrors().addItemError(getFieldName(), e);
					if(e.getStopValidation())
						// --- break validation
						break;
				}
			}
		}
	}
}
